import rhinoscriptsyntax as rs

def midPt(startPt, endPt):
    return [ (startPt[0]+endPt[0])/2, (startPt[1]+endPt[1])/2, (startPt[2]+endPt[2])/2 ]

def flattenPt(element):
    if rs.IsPoint(element): #when the point is a Point object.
        point = rs.PointCoordinates(element)
    else: #when the point is a list of numbers
        point = element
    point[2] = 0
    return point

def boundingBoxDiagonalPts(objectId):
    """Returns as an array the bottom right [0], top left [1] and centroid [2] corner points of an object's bounding box"""
    box = rs.BoundingBox(objectId,rs.NamedCPlane(projectionPlane), True) #if CPlane name is not found, falls back on WorldXY
    if box:
        bottomRightPt = [0,0,0]
        topLeftPt = [0,0,0]
        
        for i, point in enumerate(box):
            if i==1:
                bottomRightPt = point
            if i==7:
                topLeftPt = point
        centroidPt = midPt(topLeftPt,bottomRightPt)
        
        if config == [True, True]:
            return flattenPt(bottomRightPt),flattenPt(topLeftPt),flattenPt(centroidPt)
        else:
            return bottomRightPt,topLeftPt,centroidPt

duplicatesId = []
planes = []

objectIds = rs.SelectedObjects()
if len(objectIds)==0:
    objectIds = rs.GetObjects("Select objects")

tolerance = rs.GetReal("Tolerance (in Rhino units)",minimum=0.0)
configItems = ("Flatten", "False", "True"), ("0","0","0") #useless second tuple, but Rhino won't show the boolean otherwise
config = rs.GetBoolean("Ignore Z values ?", configItems, (True, True))

if objectIds and (tolerance or tolerance == 0):
    rs.EnableRedraw(False)
    
    planes.append("WorldXY") # add World XY
    cplanes = rs.NamedCPlanes() # add named CPlanes
    
    if cplanes:
        for cplane in cplanes:
            planes.append(cplane)
    
    projectionPlane = rs.GetString("Calculation bounding boxes plane (World or named CPlanes)", "WorldXY", planes)
    
    for objectId in objectIds:
        diagonalPts = boundingBoxDiagonalPts(objectId)
        for pairObjectId in objectIds:
            if objectId != pairObjectId and pairObjectId not in duplicatesId:
                pairDiagonalPts = boundingBoxDiagonalPts(pairObjectId)
                deltaBottomRight = rs.Distance(diagonalPts[0],pairDiagonalPts[0])
                deltaTopLeft = rs.Distance(diagonalPts[1],pairDiagonalPts[1])
                if deltaBottomRight <= tolerance or deltaTopLeft <= tolerance:
                    duplicatesId.append(objectId)
                    continue
    
    rs.EnableRedraw(True)
    
    if duplicatesId != []:
        duplicatesId = list(dict.fromkeys(duplicatesId)) #removes duplicates in list
        rs.UnselectAllObjects()
        rs.SelectObjects(duplicatesId)
        print str(len(duplicatesId)) + " near duplicate(s) found."
        rs.Prompt(str(len(duplicatesId)) + " near duplicate(s) found.")
    else:
        rs.UnselectAllObjects()
        print "No near duplicates found."
        rs.Prompt("No near duplicates found.")