import rhinoscriptsyntax as rs
import scriptcontext
from base64 import b32encode
from os import urandom
import random

objects = rs.SelectedObjects()

if len(objects) == 0:
    objects = rs.GetObjects()

def getBasePoint(objects):
    if objects:
        box = rs.BoundingBox(objects)
        if box:
            for i, point in enumerate(box):
                if i == 3: #bottomLeft
                    return point
        else:
            print "Failed to create block (cannot compute bounding box)."
    else:
        print "Failed to create block. (no objects to get base point from)."

def getRandomName():
    names = ['AAPO', 'AATU', 'ABBY', 'ACKE', 'ADAN', 'ADAO', 'ADDY', 'ADEL', 'ADEM', 'ADIL', 'AIDA', 'AIME', 'AIMO', 'AINA', 'AINE', 'AINO', 'ALBY', 'ALDO', 'ALEC', 'ALES', 'ALEX', 'ALIA', 'ALIX', 'ALLY', 'ALMA', 'ALON', 'ALTE', 'ALYA', 'ALYX', 'AMIE', 'AMIN', 'ANAS', 'ANDY', 'ANKA', 'ARIA', 'ARIC', 'ARMO', 'ARON', 'ARVO', 'ARZU', 'ASAD', 'ASIA', 'ASIM', 'ASLI', 'ASMA', 'AUDE', 'AUNE', 'AURA', 'AVIS', 'AVIV', 'AZIZ', 'BABS', 'BADR', 'BAHA', 'BAKR', 'BANU', 'BARB', 'BEAU', 'BERK', 'BESS', 'BETH', 'BICE', 'BILL', 'BORA', 'BRAD', 'BRAS', 'BRET', 'BRID', 'BUCK', 'BURT', 'CADE', 'CARA', 'CARY', 'CASS', 'CATE', 'CEES', 'CHAD', 'CHAS', 'CHAZ', 'CHER', 'CHET', 'CHIP', 'CHUS', 'CIEL', 'CLAY', 'CLEM', 'CLEO', 'COLE', 'COLM', 'CONN', 'COOS', 'CORA', 'CORY', 'COTY', 'CREE', 'CRUZ', 'CUAN', 'CYAN', 'DALE', 'DANE', 'DAVE', 'DAVY', 'DAWN', 'DEAN', 'DELL', 'DEMA', 'DEMI', 'DENA', 'DENE', 'DEON', 'DICK', 'DIDI', 'DINA', 'DINO', 'DIYA', 'DONA', 'DORA', 'DOUG', 'DREW', 'DUHA', 'DUKE', 'EARL', 'EBER', 'EDDA', 'EDDY', 'EDIE', 'EERO', 'EETU', 'EEVA', 'EGON', 'EKIN', 'ELDA', 'ELEA', 'ELIA', 'ELLA', 'ELLE', 'ELLY', 'ELMO', 'ELOF', 'ELOI', 'ELOV', 'ELOY', 'EMEL', 'EMMY', 'ENYA', 'ENZO', 'EREN', 'EREZ', 'ERLE', 'ERNA', 'ERYN', 'ESER', 'ESIN', 'ESME', 'ESTA', 'ETNA', 'ETTA', 'EULA', 'EVIE', 'FAAS', 'FADI', 'FADL', 'FAHD', 'FALK', 'FAWN', 'FAYE', 'FERN', 'FIDO', 'FIFI', 'FIHR', 'FINA', 'FINN', 'FION', 'FITZ', 'FLIP', 'FORD', 'FRAN', 'FRED', 'FUAD', 'FULK', 'GABE', 'GABR', 'GAGE', 'GAIL', 'GALE', 'GALL', 'GARY', 'GAYE', 'GENA', 'GENE', 'GERT', 'GIGI', 'GIJS', 'GINO', 'GISA', 'GORD', 'GORE', 'GREG', 'GUDA', 'HADI', 'HALA', 'HALE', 'HALL', 'HANA', 'HANI', 'HANK', 'HAVA', 'HAZE', 'HEDY', 'HENA', 'HENK', 'HERB', 'HIBA', 'HIND', 'HOPE', 'HUDA', 'HUEY', 'HUGH', 'HYAM', 'IDAN', 'IHAB', 'IIDA', 'ILAN', 'ILSE', 'IMAD', 'IMAM', 'IMAN', 'IMEN', 'IMKE', 'IMMA', 'INDY', 'INES', 'INES', 'INES', 'INEZ', 'INGA', 'INTO', 'IONE', 'IRIT', 'IRJA', 'ISAK', 'ISER', 'ISRA', 'ISSA', 'IVOR', 'IWAN', 'IYOV', 'IZZY', 'JAAP', 'JABR', 'JACK', 'JADA', 'JADE', 'JAKE', 'JALO', 'JAMI', 'JANE', 'JARI', 'JAYE', 'JEFF', 'JENI', 'JENN', 'JERA', 'JERE', 'JERI', 'JERK', 'JESS', 'JILL', 'JIMI', 'JIRI', 'JOAO', 'JOBY', 'JODI', 'JODY', 'JOEL', 'JOEP', 'JOEY', 'JOOP', 'JOOS', 'JORG', 'JOSH', 'JOSS', 'JOYE', 'JUAN', 'JUDA', 'JUDD', 'JUDI', 'JUDY', 'JUHA', 'JUHO', 'JULY', 'JUNE', 'KADE', 'KANE', 'KARA', 'KATE', 'KATY', 'KAYE', 'KEAN', 'KEES', 'KEMP', 'KENT', 'KERI', 'KING', 'KIRK', 'KODY', 'KOEN', 'KOLE', 'KOOS', 'KORA', 'KORT', 'KORY', 'KUBA', 'KUNO', 'KURT', 'KYLA', 'KYLE', 'KYRA', 'LACY', 'LAKE', 'LALO', 'LANE', 'LARK', 'LEAN', 'LECH', 'LEIA', 'LENE', 'LENI', 'LENZ', 'LEON', 'LEOS', 'LETA', 'LEXA', 'LEXI', 'LEXY', 'LIAM', 'LIDA', 'LILE', 'LILI', 'LILO', 'LILY', 'LIOR', 'LISA', 'LISE', 'LITA', 'LIZA', 'LOPE', 'LORA', 'LORE', 'LORI', 'LORN', 'LOUP', 'LOVE', 'LOYD', 'LUCE', 'LUCY', 'LUDO', 'LUIZ', 'LULA', 'LULU', 'LUPE', 'LUTZ', 'LUUK', 'LYLE', 'LYNN', 'MAAS', 'MACK', 'MACY', 'MAHA', 'MARC', 'MATT', 'MAUD', 'MAVE', 'MEDB', 'MEIR', 'MERV', 'MICK', 'MIIA', 'MIKA', 'MIKE', 'MILO', 'MIMI', 'MORA', 'MORT', 'MUGE', 'MUNA', 'MYRA', 'NADA', 'NAGI', 'NAIL', 'NAJI', 'NAVA', 'NEAL', 'NELL', 'NELS', 'NENA', 'NEVE', 'NEWT', 'NICK', 'NICO', 'NIKO', 'NINO', 'NITA', 'NOAK', 'NOAM', 'NOEL', 'NOGA', 'NOLL', 'NONA', 'NOOR', 'NORM', 'NOVA', 'NUHA', 'NUNO', 'NURA', 'OFER', 'OFRA', 'OIVA', 'OLEK', 'OLLE', 'OLLI', 'OLOF', 'OLOV', 'OMAR', 'OMER', 'ONNI', 'ONUR', 'OONA', 'OPAL', 'ORAH', 'ORAN', 'OREL', 'OREN', 'ORLA', 'ORLI', 'ORLY', 'ORNA', 'ORSO', 'OTIS', 'OTSO', 'OVED', 'OZAN', 'OZZY', 'PACE', 'PAGE', 'PENE', 'PEPE', 'PETA', 'PETE', 'PETR', 'PHIL', 'PIET', 'PIIA', 'PILI', 'PINO', 'PONS', 'POSY', 'PRIS', 'PRUE', 'PYRY', 'QUIN', "RA'D", 'RAFA', 'RAFE', 'RAIN', 'RAJA', 'RALF', 'RANA', 'RAZA', 'REBA', 'REDD', 'REIN', 'REMI', 'REMO', 'REMY', 'RENA', 'RENE', 'RETO', 'REUT', 'RIAN', 'RIAZ', 'RICH', 'RICK', 'RIKE', 'RIKU', 'RINO', 'ROCH', 'ROEL', 'ROIS', 'ROLO', 'ROLY', 'RONA', 'ROSY', 'ROXY', 'RUBE', 'RUBY', 'RUDY', 'RUSS', 'RUUD', "SA'D", 'SADB', 'SAGE', 'SAKU', 'SANA', 'SANI', 'SARI', 'SATU', 'SEAN', 'SEPP', 'SERA', 'SEVE', 'SHAE', 'SHAI', 'SHAN', 'SHAW', 'SHEA', 'SHER', 'SHIR', 'SIGI', 'SILE', 'SIMO', 'SINE', 'SINI', 'SIVE', 'SKYE', 'STAN', 'STAR', 'STEF', 'STEN', 'STEW', 'SUHA', 'SULO', 'SUSE', 'SUVI', 'SUZY', 'SVEN', 'TAMI', 'TATE', 'TATU', 'TEAL', 'TERA', 'TERE', 'TERI', 'TERO', 'TESS', 'THAD', 'THEA', 'THEO', 'THEO', 'THOM', 'TILL', 'TOAL', 'TOBY', 'TODD', 'TONY', 'TOPI', 'TORI', 'TOVA', 'TREY', 'TRIX', 'TROY', 'TUNC', 'TYKO', 'UMAR', 'USKO', 'USOA', 'UZMA', 'VELI', 'VERE', 'VESA', 'VICO', 'WADE', 'WALT', 'WARD', 'WERA', 'WILF', 'WILL', 'WREN', 'XENA', 'XENE', 'YAEL', 'YOAV', 'YOEL', 'YORK', 'YRJO', 'YVES', 'ZACH', 'ZACK', 'ZAID', 'ZAIN', 'ZAKI', 'ZANE', 'ZAYD', 'ZAYN', 'ZEEV', 'ZEKE', 'ZEKI', 'ZENA', 'ZEPH', 'ZION', 'ZITA', 'ZIVA', 'ZIYA', 'ZOLA', 'ZULA', 'ZUSA']
    return random.choice(names)

def getUniqueIdef():
    bytes = b32encode(urandom(4))
    idef = bytes.decode().rstrip(b'\n=')
    idef = str(getRandomName() + "-" + idef)
    #print (idef, "is the generated instance (block) name.")
    existing_idef = scriptcontext.doc.InstanceDefinitions.Find(idef, True)
    if existing_idef:
        print (idef, "instance definition name already in use. Looking for another one.")
        return getUniqueIdef()
    else:
        return idef

def makeAliasBlock(objects):
        basePt = getBasePoint(objects)
        idef = getUniqueIdef()
        block = rs.AddBlock(objects, basePt, idef, True)
        b = rs.InsertBlock(block, basePt)
        rs.SelectObject(b)
        print "Block", idef, "added."

if objects:
    rs.EnableRedraw(False)
    #tests for just one block, will generate a new idef for this instance only (similar to "make unique")
    if len(objects) == 1 and rs.ObjectType(objects[0]) == 4096:
        explodedObjects = rs.ExplodeBlockInstance(objects[0], False)
        explodedObjects_strippedKeys = []
        for obj in explodedObjects:
            explodedObjects_strippedKeys.append(obj)
        makeAliasBlock(explodedObjects_strippedKeys)
    else:
        #One or more objects selected (but not just one block)
        makeAliasBlock(objects)
else:
    print "No object selected."